from django.core.paginator import Paginator
from django.db.models import Sum
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib import messages
from .models import Product, Profile, GeneratedImageFromTxt
import json
import stripe
from django.conf import settings
from django.views.generic import TemplateView
from django.views.decorators.csrf import csrf_exempt, csrf_protect
from django.http import JsonResponse, HttpResponse
from django.views import View
from .models import Product
# Stability SDK
from django.core.files.base import ContentFile
from pathlib import Path
import os
import io
import warnings
from PIL import Image
from stability_sdk import client
import stability_sdk.interfaces.gooseai.generation.generation_pb2 as generation
# Stability SDK
from .forms import ServiceCreateForm, GenerateImageFromTxtForm
from django.contrib.auth.models import User

import environ
# Initialise environment variables
env = environ.Env()
environ.Env.read_env()
PROJECT_NAME = env('PROJECT_NAME')


stripe.api_key = settings.STRIPE_PRIVATE_KEY


# Create your views here.
def login(request):
  return render(request, 'registration/login.html')

def index(request):

    product = Product.objects.get(custom_id=1)
    paginator = Paginator(GeneratedImageFromTxt.objects.filter(selected_for_index=True), 30)
    page_number = request.GET.get('page')
    user_generated_images = paginator.get_page(page_number)
    THIS_PROJECT_NAME = PROJECT_NAME

    if page_number == None:
        page_number = 1

    return render(request, 'index.html',{'user_generated_images':user_generated_images,
                                         "page": page_number,
                                         'product': product,
                                         'THIS_PROJECT_NAME':THIS_PROJECT_NAME,
                                         })

def PrivacyPolicy(request):
    WEBSITE_DOMAIN = env('WEBSITE_DOMAIN')
    THIS_PROJECT_NAME = PROJECT_NAME

    return render(request, 'privacy-policy.html',{
        "WEBSITE_DOMAIN":WEBSITE_DOMAIN,
        'THIS_PROJECT_NAME': THIS_PROJECT_NAME,
    })



@login_required
def TextToImage(request):
    product = Product.objects.get(custom_id=1)
    user_credits = Profile.objects.filter(user=request.user).aggregate(user_credits=Sum('credits'))['user_credits']
    # user_generated_images = GeneratedImageFromTxt.objects.filter(profile = request.user.profile.id)
    paginator = Paginator(GeneratedImageFromTxt.objects.filter(profile = request.user.profile.id), 30)
    page_number = request.GET.get('page')
    user_generated_images = paginator.get_page(page_number)
    product_price = product.price / 100
    images_text = GeneratedImageFromTxt.objects.filter(selected_for_index=True).order_by('?')
    THIS_PROJECT_NAME = PROJECT_NAME

    if page_number == None:
        page_number = 1

    if request.method == "POST" :
        # if request.POST.get("submit") == 'submit':
        gentext = request.POST['gentext']


        # Our Host URL should not be prepended with "https" nor should it have a trailing slash.
        os.environ['STABILITY_HOST'] = 'grpc.stability.ai:443'

        # Sign up for an account at the following link to get an API Key.
        # https://beta.dreamstudio.ai/membership

        # Click on the following link once you have created an account to be taken to your API Key.
        # https://beta.dreamstudio.ai/membership?tab=apiKeys

        # Paste your API Key below.

        os.environ['STABILITY_KEY'] = env('STABILITY_KEY')

        # Set up our connection to the API.
        stability_api = client.StabilityInference(
            key=os.environ['STABILITY_KEY'],  # API Key reference.
            verbose=True,  # Print debug messages.
            engine="stable-diffusion-v1-5",  # Set the engine to use for generation.
            # Available engines: stable-diffusion-v1 stable-diffusion-v1-5 stable-diffusion-512-v2-0 stable-diffusion-768-v2-0
            # stable-diffusion-512-v2-1 stable-diffusion-768-v2-1 stable-inpainting-v1-0 stable-inpainting-512-v2-0
        )

        try:
            # Set up our initial generation parameters.
            answers = stability_api.generate(
                prompt=gentext,
                # seed=992446758,  # If a seed is provided, the resulting generated image will be deterministic.
                # What this means is that as long as all generation parameters remain the same, you can always recall the same image simply by generating it again.
                # Note: This isn't quite the case for Clip Guided generations, which we'll tackle in a future example notebook.
                steps=30,  # Amount of inference steps performed on image generation. Defaults to 30.
                cfg_scale=18.0,  # Influences how strongly your generation is guided to match your prompt.
                # Setting this value higher increases the strength in which it tries to match your prompt.
                # Defaults to 7.0 if not specified.
                width=512,  # Generation width, defaults to 512 if not included.
                height=512,  # Generation height, defaults to 512 if not included.
                samples=1,  # Number of images to generate, defaults to 1 if not included.
                sampler=generation.SAMPLER_K_DPMPP_2M  # Choose which sampler we want to denoise our generation with.
                # Defaults to k_dpmpp_2m if not specified. Clip Guidance only supports ancestral samplers.
                # (Available Samplers: ddim, plms, k_euler, k_euler_ancestral, k_heun, k_dpm_2, k_dpm_2_ancestral, k_dpmpp_2s_ancestral, k_lms, k_dpmpp_2m)
            )

            # Set up our warning to print to the console if the adult content classifier is tripped.
            # If adult content classifier is not tripped, save generated images.
            for resp in answers:
                for artifact in resp.artifacts:
                    if artifact.finish_reason == generation.FILTER:
                        warnings.warn(
                            "Your request activated the API's safety filters and could not be processed."
                            "Please modify the prompt and try again.")
                    if artifact.type == generation.ARTIFACT_IMAGE:
                        img = Image.open(io.BytesIO(artifact.binary))

                        generated_img = GeneratedImageFromTxt.objects.create(txt=gentext, profile=Profile.objects.filter(user=request.user).order_by('id').last())
                        generated_img.generated_img.save(name="Img2Img.png", content=io.BytesIO(artifact.binary),save=True)
                        user = request.user
                        user.profile.credits = user.profile.credits - 1
                        user.profile.save()
                        # img.save(
                        #     str(artifact.seed) + ".png")  # Save our generated images with their seed number as the filename.
        except:
            messages.error(request, 'The prompt contains an invalid keyword!')
            return redirect('TextToImage')

        return redirect('TextToImage')

    return render(request, 'text-to-image.html',{
                                                         'form': GenerateImageFromTxtForm(request.POST),
                                                         # 'user_images':user_images,
                                                         'user_generated_images':user_generated_images,
                                                         'user_credits':user_credits,
                                                         'product':product,
                                                         'product_price':product_price,
                                                         "page": page_number,
                                                         'images_text':images_text,
                                                         "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
                                                         'THIS_PROJECT_NAME': THIS_PROJECT_NAME,
                                                         })

def download_image_from_txt(request, image_id):
    # Get the image object using the image_id
    image = GeneratedImageFromTxt.objects.get(id=image_id)

    # Open the image file
    with open(image.generated_img.path, 'rb') as f:
        # Create an HttpResponse object with the image file as the content
        response = HttpResponse(f.read(), content_type="image/jpeg")

    # Set the content disposition header to "attachment" so that the browser prompts the user to download the file
    response['Content-Disposition'] = 'attachment; filename=%s' % image.generated_img.name

    return response

def remove_image_generated_from_text(request, id):
    image = GeneratedImageFromTxt.objects.filter(id = id)
    image.delete()
    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))


def add_to_frontpage_image_generated_from_text(request, id):
    image = GeneratedImageFromTxt.objects.filter(id = id).last()
    image.selected_for_index = True
    image.save()
    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

def remove_from_frontpage_image_generated_from_text(request, id):
    image = GeneratedImageFromTxt.objects.filter(id = id).last()
    image.selected_for_index = False
    image.save()
    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))


@login_required
def top_up(request):
    product = Product.objects.get(custom_id=2)
    product2 = Product.objects.get(custom_id=3)
    product_price = product.price / 100
    product_price2 = product2.price / 100
    user_credits = Profile.objects.filter(user=request.user).aggregate(user_credits=Sum('credits'))['user_credits']
    # print(user_credits)
    #
    if user_credits > 0:
        return redirect("TextToImageOpenJourney")
    return render(request, 'subscription-pricing.html',{
        'product':product,
        'product2':product2,
        'product_price':product_price,
        'product_price2':product_price2,
        "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
    })

def is_admin(user):
    return user.is_staff

@login_required
@user_passes_test(is_admin)
def images_control(request):
    paginator = Paginator(GeneratedImageFromTxt.objects.all(), 30)
    page_number = request.GET.get('page')
    user_generated_images = paginator.get_page(page_number)
    THIS_PROJECT_NAME = PROJECT_NAME
    product = Product.objects.get(custom_id=1)

    return render(request, 'images-control.html',{'user_generated_images':user_generated_images,
                                                  'THIS_PROJECT_NAME': THIS_PROJECT_NAME,
                                                  "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
                                                  "product": product,
                                                  "page": page_number,})

def SuccessView(request):
    product = Product.objects.get(custom_id=1)
    user_credits = Profile.objects.filter(user=request.user).aggregate(user_credits=Sum('credits'))['user_credits']
    product_price = product.price / 100

    return render(request, 'success.html', {

        "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
        'user_credits': user_credits,
        "product": product,
        'product_price': product_price,
    })

def CancelView(request):
    product = Product.objects.get(custom_id=1)
    user_credits = Profile.objects.filter(user=request.user).aggregate(user_credits=Sum('credits'))['user_credits']
    product_price = product.price / 100

    return render(request, 'cancel.html', {

        "STRIPE_PUBLIC_KEY": settings.STRIPE_PUBLIC_KEY,
        'user_credits': user_credits,
        "product":product,
        'product_price':product_price,
    })


# Payments
class CreateCheckoutSessionView(View):
    def post(self, request, *args, **kwargs):
        product_id = self.kwargs["pk"]
        product = Product.objects.get(id=product_id)
        YOUR_DOMAIN = env('WEBSITE_URL_MY')
        print(type(YOUR_DOMAIN))
        user = self.request.user
        checkout_session = stripe.checkout.Session.create(
            payment_method_types=['card'],
            line_items=[
                {
                    'price_data': {
                        'currency': 'usd',
                        'unit_amount': product.price,
                        'product_data': {
                            'name': product.name,
                            # 'images': ['https://i.imgur.com/EHyR2nP.png'],
                        },
                    },
                    'quantity': 1,

                },
            ],
            metadata={
                "product_id": product.id,
                "product":product,
                'user_id':user.id,
            },
            mode='payment',
            # success_url=YOUR_DOMAIN + '/success/',
            # cancel_url=YOUR_DOMAIN + '/cancel/',
            success_url=YOUR_DOMAIN + '/text-to-image',
            cancel_url=YOUR_DOMAIN + '/text-to-image',
        )
        return JsonResponse({
            'id': checkout_session.id
        })

@csrf_exempt
def stripe_webhook(request):
    payload = request.body
    sig_header = request.META['HTTP_STRIPE_SIGNATURE']
    event = None

    try:
        event = stripe.Webhook.construct_event(
            payload, sig_header, settings.STRIPE_WEBHOOK_SECRET
        )
    except ValueError as e:
        # Invalid payload
        print("Invalid payload")
        return HttpResponse(status=400)
    except stripe.error.SignatureVerificationError as e:
        # Invalid signature
        print("Invalid signature")
        return HttpResponse(status=400)

    # Handle the checkout.session.completed event
    if event['type'] == 'checkout.session.completed':
        session = event['data']['object']

        customer_email = session["customer_details"]["email"]
        product_id = session["metadata"]["product_id"]
        user_id = session["metadata"]["user_id"]

        user = User.objects.get(id=user_id)
        product = Product.objects.get(id=product_id)
        # print("Laba diena")
        # print(product.coins)
        user.profile.credits = user.profile.credits + product.coins
        user.profile.spend = user.profile.spend + (product.price / 100)
        user.profile.save()


        # send_mail(
        #     subject="Here is your product",
        #     message=f"Thanks for your purchase. Here is the product you ordered. The URL is {product.url}",
        #     recipient_list=[customer_email],
        #     from_email="matt@test.com"
        # )

        # TODO - decide whether you want to send the file or the URL

    # elif event["type"] == "payment_intent.succeeded":
    #     intent = event['data']['object']
    #
    #     stripe_customer_id = intent["customer"]
    #     stripe_customer = stripe.Customer.retrieve(stripe_customer_id)
    #
    #     customer_email = stripe_customer['email']
    #     product_id = intent["metadata"]["product_id"]
    #
    #     product = Product.objects.get(id=product_id)
    #
    #     send_mail(
    #         subject="Here is your product",
    #         message=f"Thanks for your purchase. Here is the product you ordered. The URL is {product.url}",
    #         recipient_list=[customer_email],
    #         from_email="matt@test.com"
    #     )

    return HttpResponse(status=200)


class StripeIntentView(View):
    def post(self, request, *args, **kwargs):
        try:
            req_json = json.loads(request.body)
            customer = stripe.Customer.create(email=req_json['email'])
            product_id = self.kwargs["pk"]
            product = Product.objects.get(id=product_id)
            intent = stripe.PaymentIntent.create(
                amount=product.price,
                currency='usd',
                customer=customer['id'],
                metadata={
                    "product_id": product.id
                }
            )
            return JsonResponse({
                'clientSecret': intent['client_secret']
            })
        except Exception as e:
            return JsonResponse({'error': str(e)})